#pragma once

/* Genode includes */
#include <session/session.h>
#include <base/rpc.h>
#include <dataspace/capability.h>

namespace Check_certificate
{
	using Genode::size_t;

	struct Session;

	class Buffer_too_small : public Genode::Exception { };
}


struct Check_certificate::Session : Genode::Session
{
	/**
	 * \noapi
	 */
	static const char *service_name() { return "Check_certificate"; }

	/*
	 * A Check_certificate session consumes a dataspace capability for the server's
	 * session-object allocation, its session capability, and a dataspace
	 * capability for the communication buffer.
	 */
	enum { CAP_QUOTA = 3 };

	/**
	 * Signs a piece of data, most probably a hash.
	 *
	 * \param  data_buffer  pointer to buffer containing the certificate to be checked.
	 * \param  data_bytes   number of bytes in the buffer.
	 *
	 * \result  true if certificate is ok, false otherwise.
	 */
	virtual bool check_certificate(const char* data_buffer, size_t data_bytes) = 0;

	/*******************
	 ** RPC interface **
	 *******************/

	GENODE_RPC_THROW(Rpc_check_certificate, bool,   _check_certificate, GENODE_TYPE_LIST(Buffer_too_small), size_t);
	GENODE_RPC(Rpc_dataspace, Genode::Dataspace_capability, _dataspace);

	GENODE_RPC_INTERFACE(Rpc_check_certificate, Rpc_dataspace);
};

