#pragma once

// Genode includes
#include <util/misc_math.h>
#include <util/string.h>
#include <base/lock.h>
#include <base/rpc_client.h>
#include <base/attached_dataspace.h>

// gapfruit includes
#include <check_certificate_session/check_certificate_session.h>


namespace Check_certificate
{
	using Genode::size_t;
	using Genode::uint8_t;
	using Genode::memcpy;

	class Session_client;
}

class Check_certificate::Session_client : public Genode::Rpc_client<Session>
{
	private:

		Genode::Lock _lock { };

		/**
		 * Shared-memory buffer used for carrying the payload
		 * of read/write operations
		 */
		Genode::Attached_dataspace _io_buffer;

	public:

		Session_client(Genode::Region_map &local_rm, Genode::Capability<Session> cap)
		:
			Genode::Rpc_client<Session>(cap),
			_io_buffer(local_rm, call<Rpc_dataspace>())
		{ }

		bool check_certificate(const char* data_buffer, Genode::size_t data_bytes) override
		{
			Genode::Lock::Guard _guard(_lock);

			if (data_bytes > _io_buffer.size()) throw Buffer_too_small();

			memcpy(_io_buffer.local_addr<char>(), data_buffer, data_bytes);

			return call<Rpc_check_certificate>(data_bytes);
		}
};
